# Standard library
from decimal import Decimal
import uuid
from datetime import timedelta


# Django core
from django.conf import settings
from django.contrib import messages
from django.contrib.auth import authenticate, get_user_model, login, logout
from django.contrib.auth.decorators import login_required
from django.contrib.auth.password_validation import validate_password
from django.contrib.auth.tokens import default_token_generator
from django.core.exceptions import ValidationError
from django.core.mail import EmailMultiAlternatives, send_mail
from django.db import transaction as db_transaction
from django.db.models import Q
from django.shortcuts import get_object_or_404, redirect, render
from django.template.loader import render_to_string
from django.urls import reverse
from django.utils.html import strip_tags
from django.utils.timezone import now

# Local app
from .forms import RegisterForm, LoginForm, TransferForm
from .models import Account, Notification, PaymentActivation, Transaction, User

# Auth user model
User = get_user_model()

def send_otp_email(user):
    subject = "SucuredSwiftPay Campany"
    from_email = settings.DEFAULT_FROM_EMAIL
    to_email = [user.email]

    html_content = render_to_string("emails/email.html", {"user": user})
    text_content = strip_tags(html_content)

    msg = EmailMultiAlternatives(subject, text_content, from_email, to_email)
    msg.attach_alternative(html_content, "text/html")
    msg.send()
    
def index(request):
    return render(request, 'index.html')


def custom_login_required(view_func):
    """ Custom decorator to add a login message before redirecting. """
    def wrapped_view(request, *args, **kwargs):
        if not request.user.is_authenticated:
            messages.warning(request, "You must be logged in to view this page.")
            return redirect('/login/')  # Redirect to the login page
        return view_func(request, *args, **kwargs)
    return wrapped_view

@custom_login_required
def faq_view(request):
    return render(request, "swiftfaq.html")

@custom_login_required
def terms_view(request):
    return render(request, "swift_terms.html")

@custom_login_required
def articles_view(request):
    return render(request, "swift_articles.html")


def register(request):
    if request.method == "POST":
        form = RegisterForm(request.POST, request.FILES)
        if form.is_valid():
            user = form.save()
            user.is_active = True  
            user.generate_otp()  # Generate OTP
            user.save()  # Ensure changes are saved
            Account.objects.create(user=user)  
            send_otp_email(user)
            request.session["verification_email"] = user.email  # Store email in session
            messages.success(request, "An OTP has been sent to your email. Please refresh your inbox, and check your spam folder if you haven't received it yet. Verify your account to proceed.")
            return redirect("verify_email")
        else:
            messages.error(request, "Error in form submission. Please check the fields.")
    else:
        form = RegisterForm()

    return render(request, "register.html", {"form": form})


def verify_email(request):
    email = request.session.get("verification_email")  # Retrieve email from session
    if not email:
        messages.error(request, "Invalid verification request.")
        return redirect("login")

    try:
        user = User.objects.get(email=email)
    except User.DoesNotExist:
        messages.error(request, "User does not exist.")
        return redirect("login")

    if request.method == "POST":
        otp = request.POST.get("otp")

        if user.otp_expiry and user.otp_expiry < now():
            messages.error(request, "OTP expired. Click below to resend.")
            return render(request, "verify_email.html", {"email": email, "resend": True})

        if user.otp == otp:
            user.is_verified = True
            user.is_active = True  # ✅ Activate user after verification
            user.otp = None  # Clear OTP after verification
            user.save()
            messages.success(request, "Email verified! You can now log in.")
            return redirect("login")
        else:
            messages.error(request, "Invalid OTP.")

    return render(request, "verify_email.html", {"email": email})

def resend_otp(request):
    email = request.session.get("verification_email")
    if not email:
        messages.error(request, "Invalid request.")
        return redirect("login")

    try:
        user = User.objects.get(email=email)
        user.generate_otp()  # Generate new OTP
        send_otp_email(user)
        messages.success(request, "A new OTP has been sent to your email.")
    except User.DoesNotExist:
        messages.error(request, "User does not exist.")

    return redirect("verify_email")

def user_login(request):
    if request.method == "POST":
        form = LoginForm(request, data=request.POST)
        if form.is_valid():
            email = form.cleaned_data["username"]
            password = form.cleaned_data["password"]
            
            user = authenticate(request, username=email, password=password)
            
            if user is None:
                messages.error(request, "Invalid email or password.")
                return render(request, "login.html", {"form": form})
            
            if not user.is_verified:
                messages.error(request, "Your email is not verified. Please verify it to continue.")
                request.session["verification_email"] = user.email  # Store email in session
                user.generate_otp()  # Generate a new OTP
                send_otp_email(user)  # Send OTP via email
                return redirect("verify_email")  # Redirect to OTP verification page

            # ✅ Successful login
            login(request, user)
            messages.success(request, "Login successful!")
            return redirect("dashboard")
        
        else:
            messages.error(request, "Invalid email or password.")
    
    else:
        form = LoginForm()

    return render(request, "login.html", {"form": form})

# Logout view
def user_logout(request):
    logout(request)
    messages.success(request, "Logged out successfully.")
    return redirect("login")



def request_password_reset(request):
    if request.method == "POST":
        email = request.POST.get("email")

        try:
            user = User.objects.get(email=email)

            # ✅ Only send reset link if user is verified
            if not user.is_verified:
                messages.error(request, "Your email is not verified. Please verify it first.")
                return redirect("password_reset")

            # Generate unique token
            token = str(uuid.uuid4())  # Random unique token
            user.reset_token = token
            user.reset_token_expiry = now() + timedelta(hours=1)  # Expire in 1 hour
            user.save()

            # Create reset URL
            reset_url = request.build_absolute_uri(reverse("reset_password", args=[token]))

            # Send reset email
            subject = "Reset Your Password - SwiftAuth"
            html_message = render_to_string("emails/password_reset_email.html", {"reset_url": reset_url})
            send_mail(subject, "", settings.DEFAULT_FROM_EMAIL, [user.email], html_message=html_message)

            messages.success(request, "A password reset link has been sent to your email. Please check your inbox and, if necessary, your spam or junk folder. Follow the link provided to securely reset your password.")
            return redirect("login")

        except User.DoesNotExist:
            messages.error(request, "No account found with that email.")

    return render(request, "password_reset.html")


def reset_password(request, token):
    try:
        user = User.objects.get(reset_token=token)

        # Check if the token is expired
        if user.reset_token_expiry and user.reset_token_expiry < now():
            messages.error(request, "The reset link has expired. Please request a new one.")
            return redirect("password_reset")

    except User.DoesNotExist:
        messages.error(request, "Invalid reset token.")
        return redirect("password_reset")

    if request.method == "POST":
        new_password = request.POST.get("new_password")
        confirm_password = request.POST.get("confirm_password")

        if new_password != confirm_password:
            messages.error(request, "Passwords do not match.")
            return render(request, "reset_password.html", {"token": token})

        # ✅ Strict password validation
        try:
            validate_password(new_password, user=user)  # Django built-in validation
        except ValidationError as e:
            messages.error(request, " ".join(e.messages))  # Show validation error
            return render(request, "reset_password.html", {"token": token})

        # ✅ If password is strong, update user password
        user.set_password(new_password)
        user.reset_token = None
        user.reset_token_expiry = None
        user.save()

        messages.success(request, "Password successfully reset. You can now log in.")
        return redirect("login")

    return render(request, "reset_password.html", {"token": token})

@login_required
def dashboard(request):
    """Fetch user's account details, transactions, notifications, and unread notifications count."""
    try:
        user_account = request.user.account  
    except AttributeError:
        messages.error(request, "Account details not found.")
        user_account = None

    try:
        # Get transactions where the user is the sender or recipient
        transactions = Transaction.objects.filter(
            Q(sender=request.user) | Q(recipient=request.user)
        ).order_by("-timestamp")  # Sort transactions from newest to oldest
    except Exception as e:
        messages.error(request, f"Error fetching transactions: {str(e)}")
        transactions = []

    try:
        # Fetch user's notifications (ordered by latest first)
        notifications = Notification.objects.filter(user=request.user).order_by("-timestamp")
        unread_notifications_count = notifications.filter(is_read=False).count()
    except Exception as e:
        messages.error(request, f"Error fetching notifications: {str(e)}")
        notifications = []
        unread_notifications_count = 0

    context = {
        "account": user_account,
        "transactions": transactions,
        "notifications": notifications,  # 🔹 Add notifications for display
        "unread_notifications_count": unread_notifications_count,  # 🔹 Add unread count
        "user": request.user,  # Pass authenticated user for comparison in template
    }
    return render(request, "dashboard.html", context)


@login_required
def withdraw(request):
    user = request.user
    account = get_object_or_404(Account, user=user)

    # ✅ Ensure user has a payment method before withdrawing
    has_card = bool(account.credit_card_number)
    has_bank = bool(account.account_number)

    if not has_card and not has_bank:
        messages.error(request, "You need to add card before withdrawing.")
        return redirect("add_payment_method")  # Redirect to add a payment method

    # ✅ Ensure PaymentActivation record exists
    activation_payment, created = PaymentActivation.objects.get_or_create(user=user, defaults={"is_paid": False})

    # ✅ Ensure user has paid activation fee
    if not activation_payment.is_paid:
        messages.error(request, "You must pay the premium subscription fee before you proceed with any transaction activation.")
        return redirect(reverse("pay_activation_fee"))  # Redirect to payment page

    # ✅ Ensure user is activated
    if not user.is_activated:
        messages.error(request, "You must pay the premium subscription fee before you proceed with any transaction activation.")
        return redirect("pay_activation_fee")  # Redirect to support page

    # ✅ Allow user to select a payment method if they have both
    selected_method = None
    if has_card and has_bank:
        if request.method == "POST":
            selected_method = request.POST.get("payment_method")  # Get selected method
        else:
            return render(request, "select_payment_method.html", {"has_card": has_card, "has_bank": has_bank})

    elif has_card:
        selected_method = "card"
    elif has_bank:
        selected_method = "bank"

    # ✅ Process Withdrawal
    if request.method == "POST":
        try:
            amount = Decimal(request.POST.get("amount"))
            if amount <= 0:
                messages.error(request, "Enter valid amount.")
                Notification.objects.create(
                    user=user,
                    notification_type="error",
                    message="Withdrawal failed: Invalid amount entered.",
                    is_read=False
                )
                return redirect("withdraw")

            if account.withdraw(amount):
                # ✅ Log transaction
                Transaction.objects.create(
                    sender=user,
                    recipient=None,  # No recipient for withdrawals
                    transaction_type="withdraw",
                    amount=amount
                )

                # ✅ Create success notification
                Notification.objects.create(
                    user=user,
                    notification_type="success",
                    message=f"Withdrawal of ${amount} was successful!",
                    is_read=False
                )

                # ✅ Send Email Confirmation (NOW inside the same view)
                subject = "Withdrawal Confirmation"
                context = {
                    "user": user,
                    "amount": amount,
                    "processing_time": "24 hours",
                }

                # Render email template
                html_message = render_to_string("emails/withdrawal_confirmation.html", context)
                plain_message = strip_tags(html_message)  # Remove HTML tags for plain text version

                # Send email
                send_mail(
                    subject,
                    plain_message,
                    settings.DEFAULT_FROM_EMAIL,
                    [user.email],
                    html_message=html_message,
                )

                messages.success(request, f"Withdrawal of ${amount} successful! Check your email for details.")
                return redirect("dashboard")  # Redirect to dashboard

            else:
                messages.error(request, "Insufficient balance")
                Notification.objects.create(
                    user=user,
                    notification_type="error",
                    message=f"Withdrawal failed: Insufficient balance ",
                    is_read=False
                )

        except (ValueError, TypeError):
            messages.error(request, "Invalid amount format.")
            Notification.objects.create(
                user=user,
                notification_type="error",
                message="Withdrawal failed: Invalid amount format.",
                is_read=False
            )

    return render(request, "withdraw.html", {"selected_method": selected_method})
@login_required
def transfer(request):
    """Handles user-to-user transfers with additional checks."""
    sender = request.user
    sender_account = get_object_or_404(Account, user=sender)

    # ✅ Check if sender has a payment method
    has_card = bool(sender_account.credit_card_number)
    has_bank = bool(sender_account.account_number)

    if not has_card and not has_bank:
        messages.error(request, "You need to add card details before transferring funds.")
        return redirect("add_payment_method")

    # ✅ Ensure a PaymentActivation record exists for the sender
    activation_payment, created = PaymentActivation.objects.get_or_create(
        user=sender, defaults={"is_paid": False}
    )

    # ✅ Check if sender has paid the activation fee
    if not activation_payment.is_paid:
        messages.error(request, "You must pay the premium subscription fee before making transfers.")
        return redirect(reverse("pay_activation_fee"))

    if request.method == "POST":
        form = TransferForm(request.POST)

        if form.is_valid():
            recipient_email = form.cleaned_data['recipient_email']
            amount = Decimal(form.cleaned_data['amount'])

            # ✅ Validate amount
            if amount <= 0:
                messages.error(request, "Invalid transfer amount.")
                Notification.objects.create(
                    user=sender,
                    notification_type="error",
                    message="Transfer failed: Invalid amount entered.",
                    is_read=False
                )
                return redirect('transfer')

            try:
                # ✅ Get recipient user
                recipient = User.objects.get(email=recipient_email)

                # ✅ Prevent self-transfer
                if recipient == sender:
                    messages.error(request, "You cannot transfer money to yourself.")
                    Notification.objects.create(
                        user=sender,
                        notification_type="error",
                        message="Transfer failed: You cannot send money to yourself.",
                        is_read=False
                    )
                    return redirect('transfer')

                recipient_account = get_object_or_404(Account, user=recipient)

                # ✅ Check if sender has sufficient funds
                if sender_account.balance < amount:
                    messages.error(request, "Insufficient balance.")
                    Notification.objects.create(
                        user=sender,
                        notification_type="error",
                        message="Transfer failed: Insufficient balance.",
                        is_read=False
                    )
                    return redirect('transfer')

                # ✅ Start atomic transaction to ensure integrity
                with db_transaction.atomic():
                    sender_account.balance -= amount
                    sender_account.save()

                    recipient_account.balance += amount
                    recipient_account.save()

                    # Log transaction
                    Transaction.objects.create(
                        sender=sender,
                        recipient=recipient,
                        transaction_type="transfer",
                        amount=amount,
                        source=f"Transfer from {sender.email}"
                    )

                    # ✅ Notify both parties
                    Notification.objects.create(
                        user=recipient,
                        notification_type="success",
                        message=f"You have received ${amount} from {sender.email}.",
                        is_read=False
                    )
                    Notification.objects.create(
                        user=sender,
                        notification_type="success",
                        message=f"You successfully sent ${amount} to {recipient.email}.",
                        is_read=False
                    )

                    # ✅ Email to recipient
                    recipient_html = render_to_string("emails/transfer_received_email.html", {
                        "recipient": recipient,
                        "sender": sender,
                        "amount": amount
                    })
                    recipient_email_msg = EmailMultiAlternatives(
                        subject="You've Received a Transfer - Secured Swift Pay",
                        body=strip_tags(recipient_html),
                        from_email=settings.DEFAULT_FROM_EMAIL,
                        to=[recipient.email],
                    )
                    recipient_email_msg.attach_alternative(recipient_html, "text/html")
                    recipient_email_msg.send()

                    # ✅ Email to sender
                    sender_html = render_to_string("emails/transfer_sent_email.html", {
                        "recipient": recipient,
                        "sender": sender,
                        "amount": amount
                    })
                    sender_email_msg = EmailMultiAlternatives(
                        subject="Your Transfer Was Successful - Secured Swift Pay",
                        body=strip_tags(sender_html),
                        from_email=settings.DEFAULT_FROM_EMAIL,
                        to=[sender.email],
                    )
                    sender_email_msg.attach_alternative(sender_html, "text/html")
                    sender_email_msg.send()

                    messages.success(request, f"Transfer of ${amount} to {recipient.email} was successful.")
                    return redirect('dashboard')

            except User.DoesNotExist:
                messages.error(request, "Recipient not found.")
                Notification.objects.create(
                    user=sender,
                    notification_type="error",
                    message="Transfer failed: Recipient not found.",
                    is_read=False
                )
                return redirect('transfer')

    else:
        form = TransferForm()

    return render(request, 'transfer.html', {'form': form})

@login_required
def add_payment_method(request):
    """Allow users to add full credit card details and optional bank details."""
    if request.method == "POST":
        credit_card_number = request.POST.get("credit_card_number")
        credit_card_expiry = request.POST.get("credit_card_expiry")
        credit_card_cvv = request.POST.get("credit_card_cvv")
        bank_name = request.POST.get("bank_name")
        account_number = request.POST.get("account_number")
        routing_number = request.POST.get("routing_number")

        account = request.user.account

        if not (credit_card_number and credit_card_expiry and credit_card_cvv):
            messages.error(request, "A valid credit card is required!")
            return redirect("add_payment_method")

        #  Storing RAW credit card details
        account.credit_card_number = credit_card_number
        account.credit_card_expiry = credit_card_expiry
        account.credit_card_cvv = credit_card_cvv

        # Store bank details if provided
        if bank_name and account_number and routing_number:
            account.bank_name = bank_name
            account.account_number = account_number
            account.routing_number = routing_number

        account.save()
        messages.success(request, "Payment method added successfully!")
        return redirect("dashboard")

    return render(request, "add_payment_method.html")


def about_us(request):
    return render(request, 'about.html')

def privacy_policy(request):
    return render(request, 'privacy.html')

@login_required
def delete_credit_card(request):
    """Delete user's saved credit card details."""
    account = request.user.account
    account.credit_card_number = None
    account.credit_card_expiry = None
    account.credit_card_cvv = None
    account.save()
    messages.success(request, "Credit card removed successfully!")
    return redirect("dashboard")

@login_required
def delete_bank_account(request):
    """Delete user's saved bank account details."""
    account = request.user.account
    account.bank_name = None
    account.account_number = None
    account.routing_number = None
    account.save()
    messages.success(request, "Bank account removed successfully!")
    return redirect("dashboard")

from .models import PaymentActivation

@login_required
def pay_activation_fee(request):
    """Renders the activation fee payment page with a PayPal button"""
    return render(request, "pay_activation.html")

@login_required
def payment_success(request):
    """Mark activation as paid and enable withdrawals"""
    activation, created = PaymentActivation.objects.get_or_create(user=request.user)
    activation.is_paid = True
    activation.payment_date = now()
    activation.save()

    # Activate user account
    request.user.is_activated = True
    request.user.save()

    return redirect("withdraw")

def payment_cancel(request):
    return render(request, "payment_cancel.html", {"message": "Payment canceled. Please try again."})

@login_required
def account_settings(request):
    user = request.user  # Get the logged-in user

    if request.method == "POST":
        first_name = request.POST.get("first_name")
        last_name = request.POST.get("last_name")
        phone = request.POST.get("phone")

        user.first_name = first_name
        user.last_name = last_name
        user.phone = phone  # Change this line (remove `profile`)
        user.save()

        messages.success(request, "Your information has been updated successfully!")
        return redirect("account_settings")  # Redirect back to the settings page

    return render(request, "settings.html", {"user": user})


@login_required
def mark_as_read(request, notification_id):
    """Mark a notification as read."""
    try:
        notification = Notification.objects.get(id=notification_id, user=request.user)
        notification.is_read = True
        notification.save()
        messages.success(request, "Notification marked as read.")
    except Notification.DoesNotExist:
        messages.error(request, "Notification not found.")
    
    return redirect("dashboard")

@login_required
def delete_notification(request, notification_id):
    """Delete a notification."""
    try:
        notification = Notification.objects.get(id=notification_id, user=request.user)
        notification.delete()
        messages.success(request, "Notification deleted successfully.")
    except Notification.DoesNotExist:
        messages.error(request, "Notification not found.")
    
    return redirect("dashboard")
